// LightEmulator_1_01
//
// Version 1.0.1
// Updated on 2015.03.20
//
// - Mode EPC=0xB6, 0x41: AUTO, 0x42(66):通常灯, 0x43(67):常夜灯, 0x45(69):カラー灯
// - Brightness implementation　EPC=0xB0, 照度レベル(%) Range:0 - 100, 0x00 - 0x64(100)
// - Color implementation  EPC=0xC0 3byte(R, G, B) range 0-255, 0x00-0xFF
// - OFF -> ON 時の動作 resume
// - node Identification number: KAIT code(0x77 + random number(8digit)
//
// Kanagawa Institute of Technology, Smart House Research Center
// Hiroyuki Fujita

import java.io.IOException;
import java.nio.ByteBuffer;
import processing.net.*;
import controlP5.*;

import com.sonycsl.echo.Echo;
import com.sonycsl.echo.node.EchoNode;
import com.sonycsl.echo.eoj.profile.NodeProfile;
import com.sonycsl.echo.eoj.device.DeviceObject;

import com.sonycsl.echo.processing.defaults.DefaultNodeProfile;
import com.sonycsl.echo.eoj.device.housingfacilities.GeneralLighting;

color backgroundColor = color(128, 128, 128);
color lightOnColor    = color(255, 255, 255);
color lightOffColor   = color(0, 0, 0);
color nightColor      = color(0x70, 0x10, 0x20);

public static final byte MODE_AUTO = 0x41;    // 点灯モード(EPC=0xB6) 自動
public static final byte MODE_NORM = 0x42;    // 点灯モード(EPC=0xB6) 通常灯
public static final byte MODE_NIGHT = 0x43;   // 点灯モード(EPC=0xB6) 常夜灯
public static final byte MODE_COLOR = 0x45;   // 点灯モード(EPC=0xB6) カラー灯

//======================== Node Profile の実装 =======================================
public class MyNodeProfile extends NodeProfile {
  
  // Profile Object Super Class properties
  byte[] mManufactureCode = { 0, 0, 0x77 }; // Manufacture code, EPC=0x8A
  byte[] mPcode = {
    (byte)0x4B ,(byte)0x41 ,(byte)0x49 ,
    (byte)0x54 ,(byte)0x2D ,(byte)0x4C ,
    (byte)0x45 ,(byte)0x4D ,(byte)0x30 ,
    (byte)0x30 ,(byte)0x31 ,(byte)0x00}; // Product code, EPC=0x8C ( KAIT-LEM001 )
  byte[] mDate = {0x07, (byte)0xdf, 0x03, 0x0A}; // Date of manufacture, EPC=0x8E (2015.03.10)
  
  // Node Profile Class properties
  byte[] mStatus = { 0x30 };        // Operating status : (EPC=0x80, EDT 0x30:ON, 0x31:OFF)
  byte[] mVersion = { 1, 1, 1, 0 }; // Version infomation : EPC=0x82
  int mIdNumberFlag = 0;    // to generate mIdNumber once with random number
  byte[] mIdNumber     = { (byte)0xFE, 0, 0, (byte)0x77, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2 }; 
                                    // Identification number : EPC=0x83
  byte[] mUniqueId = { (byte)0x80, 0x01 }; // Unique identifier data : EPC=0xBF
  
  // Property map
  protected void setupPropertyMaps() {
     super.setupPropertyMaps(); // Mandatory
     // Set property map : addSetProperty();
     // Get property map : addGetProperty();
     addGetProperty(EPC_PRODUCT_CODE);        // Product code : 0x8C
     addGetProperty(EPC_DATE_OF_MANUFACTURE); // Date of manufacture : 0x8E     
     // Status change announcement property map : addStatusChangeAnnouncementProperty();
  }

  // Set, Get
  protected byte[] getManufacturerCode() { return mManufactureCode; } // Manufacture code : 0x8A
  protected byte[] getOperatingStatus() { return mStatus; }           // Operating status : 0x80
  protected byte[] getVersionInformation() { return mVersion; }       // Version infomation : 0x82
  protected byte[] getIdentificationNumber() { 
    if (mIdNumberFlag == 0) {  // set random number at bottom 8 digit
      for (int i=0; i<8; i++){
        float x = random(10);
        mIdNumber[i+9] = (byte)x;
      }
      mIdNumberFlag = 1;
    }
    println("\n mIdNumber");
    println( mIdNumber );
    return mIdNumber; }    // Identification number : 0x83
  protected byte[] getProductCode() { return mPcode; }                // Product code : 0x8C
  protected byte[] getDateOfManufacture() { return mDate; }           // Date of manufacture : 0x8E
  protected byte[] getUniqueIdentifierData() { return mUniqueId; }    // Unique identifier data : 0xBF
  protected boolean setUniqueIdentifierData(byte[] edt) {
    if ((edt[0] & 0x40) != 0x40)   return false;
    mUniqueId[0] = (byte)((edt[0] & (byte)0x7F) | (mUniqueId[0] & 0x80));
    mUniqueId[1] = edt[1];
    return true;
  }
}

//======================== 照明クラス の実装 ===================================================
public class LightEmulator extends GeneralLighting {
  byte[] mStatus = {0x31};       // 動作状態(EPC=0x80)を格納する変数です。初期値OFFを設定します。
  byte[] mLocation = {0x00};     // 設置場所(0x81)を格納する変数です。
  byte[] mFaultStatus = {0x42};  // 異常発生状態(EPC=0x88)。異常なしを設定します。
  byte[] mManufacturerCode = {0, 0, 0x77};  // メーカーコード(0x8A)。KAITのコード0x000077を設定
  byte[] mLightingMode = {MODE_NORM}; // 点灯モード(EPC=0xB6)です。通常灯(0x42)を設定します。
  byte[] mIlluminanceLevel = {0x64}; // 照度レベル(EPC=0xB0)です。100%(0x64)を設定します。
  byte[] mRGB = {127,127,127};          // RGB値
  color mlightNow = lightOffColor;
  byte[] mPcode = {    // 2015.03.20
    (byte)0x4B ,(byte)0x41 ,(byte)0x49 ,
    (byte)0x54 ,(byte)0x2D ,(byte)0x4C ,
    (byte)0x45 ,(byte)0x4D ,(byte)0x30 ,
    (byte)0x30 ,(byte)0x31 ,(byte)0x00}; // Product code, EPC=0x8C ( KAIT-LEM001 )
  int mIdNumberFlag = 0;    // 2015.03.20 to generate mIdNumber once with random number
  byte[] mIdNumber     = { (byte)0xFE, 0, 0, (byte)0x77, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2 }; 
                                    // 2015.03.20 Identification number : EPC=0x83


// プロパティリストの追加
  protected void setupPropertyMaps() {
    super.setupPropertyMaps();
    addGetProperty(EPC_ILLUMINANCE_LEVEL);
    addSetProperty(EPC_ILLUMINANCE_LEVEL);
    addGetProperty(EPC_RGB_SETTING_FOR_COLOR_LIGHTING);
    addSetProperty(EPC_RGB_SETTING_FOR_COLOR_LIGHTING);
    addGetProperty(EPC_PRODUCT_CODE);        // 2015.03.20 Product code : 0x8C
    addGetProperty(EPC_IDENTIFICATION_NUMBER);        // 2015.03.20 Product code : 0x83

  }

//============ Setter, Getter ===============================================================
//============ 機器オブジェクト スーパークラス ==================================================
// ON(0x30) / OFF(0x31)
  protected boolean setOperationStatus(byte[] edt) {
    mStatus[0] = edt[0] ;
    if(mStatus[0] == 0x30){    // ON時の処理
      if ((mLightingMode[0] == MODE_NORM)||(mLightingMode[0] == MODE_AUTO)) {
          int tmp = 255*mIlluminanceLevel[0]/100;
          mlightNow = color(tmp, tmp, tmp);
          println ("\n Power ON, 通常灯", mlightNow);
      }
      if (mLightingMode[0] == MODE_COLOR) {
          mlightNow = color(int(mRGB[0]), int(mRGB[1]), int(mRGB[2]));
          println ("\n Power ON, 通常カラー灯", mlightNow, mRGB[0], mRGB[1], mRGB[2]);
      }
      if (mLightingMode[0] == MODE_NIGHT) {
          mlightNow = nightColor;
          println ("\n Power ON, 常夜灯", mlightNow);
      }
    }
    else if (mStatus[0] == 0x31)  {  // OFF時の処理
      mlightNow = lightOffColor;
      println ("\n Power OFF, mRGB:", mRGB[0], mRGB[1], mRGB[2]);
    }

    //電源状態が変化したことを他のノードに通知します
    try { inform().reqInformOperationStatus().send(); } 
    catch (IOException e) { e.printStackTrace(); }
    return true;
  }

  protected byte[] getOperationStatus() { return mStatus; }
  
  protected boolean setInstallationLocation(byte[] edt) {
    mLocation[0] = edt[0];
    try { inform().reqInformInstallationLocation().send(); } 
    catch (IOException e) { e.printStackTrace(); }
    return true;
  }
  
  protected byte[] getInstallationLocation() { return mLocation; }
  protected byte[] getFaultStatus() { return mFaultStatus; }
  protected byte[] getManufacturerCode() { return mManufacturerCode; }
  protected byte[] getProductCode() { return mPcode; }                // 2015.03.20 Product code : 0x8C 
  protected byte[] getIdentificationNumber() { // 2015.03.20
    if (mIdNumberFlag == 0) {  // set random number at bottom 8 digit
      for (int i=0; i<8; i++){
        float x = random(10);
        mIdNumber[i+9] = (byte)x;
      }
      mIdNumberFlag = 1;
    }
    println("\n mIdNumber");
    println( mIdNumber );
    return mIdNumber; }    // Identification number : 0x83



//============ 機器オブジェクト 一般照明 ==================================================
// 点灯モード設定 (EPC=0xB6)
  protected byte[] getLightingModeSetting() { return mLightingMode; }
  protected boolean setLightingModeSetting(byte[] edt) {
      mLightingMode[0] = edt[0]; 
      if (((mLightingMode[0] == MODE_NORM)||(mLightingMode[0] == MODE_AUTO))&&(mStatus[0] == 0x30)) { // 通常灯モード
        int tmp = 255*mIlluminanceLevel[0]/100;
        mlightNow = color(tmp, tmp, tmp);
      }
      if ((mLightingMode[0] == MODE_NIGHT)&&(mStatus[0] == 0x30)) { // 常夜灯モード
        mlightNow = nightColor;
      }
      if ((mLightingMode[0] == MODE_COLOR)&&(mStatus[0] == 0x30)) { // カラー灯モード
        mlightNow = color(int(mRGB[0]), int(mRGB[1]), int(mRGB[2]));
      }
    return true;
  }

// Brightness: 照度レベル設定 (EPC=0xB0)
  protected byte[] getIlluminanceLevel() { return mIlluminanceLevel; }
  protected boolean setIlluminanceLevel(byte[] edt) {
      mIlluminanceLevel[0] = edt[0];
      println("\n Brightness: ",edt[0]);
      int tmp = 255*mIlluminanceLevel[0]/100;
      if (((mLightingMode[0] == MODE_NORM)||(mLightingMode[0] == MODE_AUTO))&&(mStatus[0] == 0x30)) { // 通常灯モード
        mlightNow = color(tmp, tmp, tmp);
      }
    return true;
 }

// Color: RGB設定 (EPC=0xC0)
  protected byte[] getRgbSettingForColorLighting() {
    return mRGB;
  }
  protected boolean setRgbSettingForColorLighting(byte[] edt) {
      mRGB = edt;
      println("\n RGB: ", int(mRGB[0]), int(mRGB[1]), int(mRGB[2]));
      if ((mLightingMode[0] == MODE_COLOR)&&(mStatus[0] == 0x30)) {      // カラー灯モード
        mlightNow = color(int(mRGB[0]), int(mRGB[1]), int(mRGB[2]));
      }
    return true;
 }
}
//======================== 照明クラス の実装 ここまで===========================================

//========================== GUI Initial ====================================================
ControlP5 cp5 ;
// 一般照明のインスタンスを１個定義します
LightEmulator light1;

//========================== Set Up ====================================================
void setup() {
// Frame Size
  size(350, 180);
  frameRate(30);
  cp5 = new ControlP5(this) ;

// フォント設定
  ControlFont cf1 = new ControlFont(createFont("Arial",16));
  ControlFont cf2 = new ControlFont(createFont("Arial",14));
  ControlFont cf3 = new ControlFont(createFont("Arial",12));
  ControlFont cf4 = new ControlFont(createFont("Arial",11));

RadioButton r1, r2;
//==================== Radio Button ON OFF ================================
  r1 = cp5.addRadioButton("radioButton_OnOff")
         .setPosition(0,5)
         .setSize(20,22)
         .setColorForeground(color(200,160,0))
         .setColorActive(color(255))
         .setColorLabel(color(255))
         .setSpacingColumn(50)
         .addItem("ON",1)
         .addItem("OFF",2)
         .activate("OFF")
         ;
     
     for(Toggle t:r1.getItems()) {
       t.captionLabel().setColorBackground(color(100,100,100));
       t.captionLabel().style().moveMargin(-2,0,0,-3);
       t.captionLabel().style().movePadding(4,0,0,3);
       t.captionLabel().style().backgroundWidth = 45;
       t.captionLabel().style().backgroundHeight = 18;
       t.captionLabel().setControlFont(cf1);
     }

//==================== Radio Button Mode ================================     
  r2 = cp5.addRadioButton("radioButton_Mode")
         .setPosition(100,5)
         .setSize(20,20)
         .setColorForeground(color(200,160,0))
         .setColorActive(color(255))
         .setColorLabel(color(255))
         .setSpacingColumn(50)
         .addItem("Norm",1)
         .addItem("Color",2)
         .addItem("Night",3)
         .activate("Norm")
         ;
     
     for(Toggle t:r2.getItems()) {
       t.captionLabel().setColorBackground(color(100,100,100));
       t.captionLabel().style().moveMargin(-4,0,0,-3);
       t.captionLabel().style().movePadding(4,0,0,3);
       t.captionLabel().style().backgroundWidth = 45;
       t.captionLabel().style().backgroundHeight = 16;
       t.captionLabel().setControlFont(cf4);
     }    

//===================== スライダー =============================
// 輝度スライダーの表示
  // name, min, max, init, posX, posY, sizeX, sizeY
  Slider s = cp5.addSlider("Brightness",0,100,100,0,75,100,20);
  s.captionLabel().setControlFont(cf3);
  
// Colorスライダーの表示
  // name, min, max, init, posX, posY, sizeX, sizeY
  Slider red = cp5.addSlider("Red",0,255,255,0,104,100,20);
  red.captionLabel().setControlFont(cf3);
  
  Slider green = cp5.addSlider("Green",0,255,0,0,127,100,20);
  green.captionLabel().setControlFont(cf3);

  Slider blue = cp5.addSlider("Blue",0,255,0,0,150,100,20);
  blue.captionLabel().setControlFont(cf3);

  // System.outにログを表示するようにします。
  Echo.addEventListener( new Echo.Logger(System.out) ) ;

  // 自分自身がLightEmulatorを含むノードになることにしましょう。
  try {
    light1 = new LightEmulator();
    light1.mRGB[0] = -0x01; light1.mRGB[1] =  0x00; light1.mRGB[2] =  0x00; // R
    Echo.start( new MyNodeProfile(), new DeviceObject[] { light1 } );
  } 
  catch( IOException e) { e.printStackTrace(); }
}

//========================== Draw ====================================================
void draw() {
  background(128);
  fill(light1.mlightNow);
  ellipse(270, 90, 90, 90);  // X pos, Y pos, X-R, Y-R
}

//========================== Radio Button "On Off" =============================================
// ON:1, OFF:2

void radioButton_OnOff(int theValue) {
//  println(" Radio Button ON OFF Event: "+theValue);
  if (theValue == 1) {
    try {
      light1.set().reqSetOperationStatus(new byte[]{0x30}).send(); // ON
//      println(" Status = ON");
    }
    catch(IOException e) { e.printStackTrace(); }
  }
  
  if (theValue == 2) {
    try {
      light1.set().reqSetOperationStatus(new byte[]{0x31}).send(); // OFF
//      println(" Status = OFF");
    }
    catch(IOException e) { e.printStackTrace(); }
  }
}

//========================== Radio Button "Mode" =============================================
// Norm:1, Color:2, Night: 3

void radioButton_Mode(int theValue) {
  println(" Radio Button Mode Event: "+theValue);

  if (theValue == 1) {      // 通常灯 Modeに設定
  try { light1.set().reqSetLightingModeSetting(new byte[]{0x42}).send(); }
  catch(IOException e) { e.printStackTrace(); }
  }
  
  else if (theValue == 2) {  // カラー灯 Modeに設定
  try { light1.set().reqSetLightingModeSetting(new byte[]{0x45}).send(); }
  catch(IOException e) { e.printStackTrace(); }
  }

  else if (theValue == 3) {  // 常夜灯Modeに設定
  try { light1.set().reqSetLightingModeSetting(new byte[]{0x43}).send(); }
  catch(IOException e) { e.printStackTrace(); }
  }
}

//========================== GUI - Brightness Slider =============================================
public void Brightness(int theValue) {
// Sliderの値をintからbyteに変換する
  byte[] tmp = fromInt(theValue);
  println(" Brightness (decimal):", tmp[3]);
  byte[] brightness = {tmp[3]};
  
    try {
      light1.set().reqSetIlluminanceLevel(brightness).send();
    }
    catch(IOException e) { e.printStackTrace(); }
}

//========================== GUI - R Slider =============================================
public void Red(int theValue) {
// Sliderの値をintからbyteに変換する
  byte[] tmp = fromInt(theValue);
  println(" SLIDER-RED int:", theValue, "byte-3:", tmp[3]);
  byte red = tmp[3];

// red の値のみ入れ替える
    try {
      byte[] colorTmp = light1.getRgbSettingForColorLighting();
      colorTmp[0] = red;
      light1.set().reqSetRgbSettingForColorLighting(colorTmp).send();
    }
    catch(IOException e) { e.printStackTrace(); }  
}

//========================== GUI - G Slider =============================================
public void Green(int theValue) {
  byte[] tmp = fromInt(theValue);
  println(" SLIDER-GREEN int:", theValue, "byte-3:", tmp[3]);
  byte green = tmp[3];

// green の値のみ入れ替える
    try {
      byte[] colorTmp = light1.getRgbSettingForColorLighting();
      colorTmp[1] = green;
      light1.set().reqSetRgbSettingForColorLighting(colorTmp).send();
    }
    catch(IOException e) { e.printStackTrace(); }  
}

//========================== GUI - B Slider =============================================
public void Blue(int theValue) {
  byte[] tmp = fromInt(theValue);
  println(" SLIDER-BLUE int:", theValue, "byte-3:", tmp[3]);
  byte blue = tmp[3];

// blue の値のみ入れ替える
    try {
      byte[] colorTmp = light1.getRgbSettingForColorLighting();
      colorTmp[2] = blue;
      light1.set().reqSetRgbSettingForColorLighting(colorTmp).send();
    }
      catch(IOException e) { e.printStackTrace(); }  
}

//========================== intをbyte[]に変換 ===========================================
public static byte[] fromInt(int value) {
        int arraySize = Integer.SIZE / Byte.SIZE;
        ByteBuffer buffer = ByteBuffer.allocate(arraySize);
        return buffer.putInt(value).array();
}
//========================== END ========================================================
